///
/// \file SoapySDR/Constants.h
///
/// Constants used in the device API.
///
/// \copyright
/// Copyright (c) 2014-2015 Josh Blum
/// SPDX-License-Identifier: BSL-1.0
///

#pragma once
#include <SoapySDR/Config.h>

/*!
 * Constant to represent the transmit direction
 */
#define SOAPY_SDR_TX 0

/*!
 * Constant to represent the receive direction
 */
#define SOAPY_SDR_RX 1

/*!
 * Indicate end of burst for transmit or receive.
 * For write, end of burst if set by the caller.
 * For read, end of burst is set by the driver.
 */
#define SOAPY_SDR_END_BURST (1 << 1)

/*!
 * Indicates that the time stamp is valid.
 * For write, the caller must set has time when timeNs is provided.
 * For read, the driver sets has time when timeNs is provided.
 */
#define SOAPY_SDR_HAS_TIME (1 << 2)

/*!
 * Indicates that stream terminated prematurely.
 * This is the flag version of an overflow error
 * that indicates an overflow with the end samples.
 */
#define SOAPY_SDR_END_ABRUPT (1 << 3)

/*!
 * Indicates transmit or receive only a single packet.
 * Applicable when the driver fragments samples into packets.
 * For write, the user sets this flag to only send a single packet.
 * For read, the user sets this flag to only receive a single packet.
 */
#define SOAPY_SDR_ONE_PACKET (1 << 4)

/*!
 * Indicate that this read call and the next results in a fragment.
 * Used when the implementation has an underlying packet interface.
 * The caller can use this indicator and the SOAPY_SDR_ONE_PACKET flag
 * on subsequent read stream calls to re-align with packet boundaries.
 */
#define SOAPY_SDR_MORE_FRAGMENTS (1 << 5)

/*!
 * Returned when read has a timeout.
 */
#define SOAPY_SDR_TIMEOUT (-1)

/*!
 * Returned for non-specific stream errors.
 */
#define SOAPY_SDR_STREAM_ERROR (-2)

/*!
 * Returned when read has data corruption.
 * For example, the driver saw a malformed packet.
 */
#define SOAPY_SDR_CORRUPTION (-3)

/*!
 * Returned when read has an overflow condition.
 * For example, and internal buffer has filled.
 */
#define SOAPY_SDR_OVERFLOW (-4)

/*!
 * Returned when a requested operation or flag setting
 * is not supported by the underlying implementation.
 */
#define SOAPY_SDR_NOT_SUPPORTED (-5)

/*!
 * Returned when a the device encountered a stream time
 * which was expired (late) or too early to process.
 */
#define SOAPY_SDR_TIME_ERROR (-6)

/*!
 * Returned when write caused an underflow condition.
 * For example, a continuous stream was interrupted.
 */
#define SOAPY_SDR_UNDERFLOW (-7)
